/* util.c
 *
 * Copyright (C) 2007-2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "util.h"


bstring bstr_map(const char* filename) {
	u_char buffer[BUFSIZ+1];
	int fd, bytes_read;
	bstring bstr;

	if (filename == NULL) {
		fprintf(stderr, "Error - Unable to map file into memory: No filename given.\n");
		exit(EXIT_FAILURE);
	}
	
	if ((fd = open(filename, O_RDONLY)) == -1) {
		fprintf(stderr, "Unable to open file %s: %m.\n", filename);
		exit(EXIT_FAILURE);
	}

	memset(&bstr, 0, sizeof(bstring));
	while ((bytes_read = read(fd, buffer, BUFSIZ)) > 0) {
		if ((bstr.data = (void *) realloc(bstr.data, bstr.len+bytes_read)) == NULL) {
			fprintf(stderr, "Unable to allocate memory: %m.\n");
			exit(EXIT_FAILURE);
		}
		memcpy(bstr.data+bstr.len, buffer, bytes_read);
		bstr.len += bytes_read;
	}
	if (bytes_read < 0) {
		fprintf(stderr, "Unable to read from %s: %m.\n", filename);
		exit(EXIT_FAILURE);
	}
	close(fd);

	return(bstr);
}


void bstr_unmap(bstring bstr) {
	free(bstr.data);
	memset(&bstr, 0, sizeof(bstring));

	return;
}


#if defined(OS_BSD) || defined (OS_DARWIN)
int regular_file(struct dirent *e) {
#else
int regular_file(const struct dirent *e) {
#endif 
	struct stat sta;
	memset(&sta, 0, sizeof(struct stat));

	if (stat(e->d_name, &sta) != 0) {
		fprintf(stderr, "Error - Unable to get file status for %s: %m.\n", e->d_name);
		exit(EXIT_FAILURE);
	}

	return(S_ISREG(sta.st_mode));
}


int timesort(const void *a, const void *b) {
	struct dirent **A, **B;
	struct stat sta, stb;
	memset(&sta, 0, sizeof(struct stat));
	memset(&stb, 0, sizeof(struct stat));

	A = (struct dirent **) a;
	B = (struct dirent **) b;

	if ((stat((*A)->d_name, &sta) != 0) || (stat((*B)->d_name, &stb) != 0)) {
		fprintf(stderr, "Error - Unable to get file status for %s, %s: %m.\n", (*A)->d_name, (*B)->d_name);
		exit(EXIT_FAILURE);
	}

	if (sta.st_ctime < stb.st_ctime) return(-1);
	if (sta.st_ctime > stb.st_ctime) return(1);
	return(0);
}
