/* avl.h
 *
 * Copyright (C) 2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _NEBULA_AVL_H
#define _NEBULA_AVL_H 1

#include <sys/types.h>


typedef struct avl_node {
	struct avl_node	*left;
	struct avl_node	*right;
	signed char	balance;
	void		*data;
	ssize_t		size;
} avl_node;


typedef struct avl_tree {
	avl_node	*root;
	ssize_t		size;
	int		(*cmpfn)(const void *a, ssize_t sizea, const void *b, ssize_t sizeb);
} avl_tree;


avl_tree *avl_tree_new(void);
void avl_free_subtree(avl_node *n);
void avl_tree_free(avl_tree *t);
void avl_tree_reset(avl_tree *t);

int avl_insert(avl_tree *t, avl_node **n, avl_node *a);
avl_node *avl_find(avl_tree *t, avl_node *n, const avl_node *a);
void avl_subtree_print(avl_node *n);

#endif
