/* cluster.h
 *
 * Copyright (C) 2007 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __NEBULA_CLUSTER_H
#define __NEBULA_CLUSTER_H 1

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <pthread.h>
#include <stdlib.h>

#include "hash.h"
#include "trie.h"
#include "queue.h"


#define	CL_WLOCK(x)	pthread_rwlock_wrlock(&((cluster *)x)->lock);
#define	CL_RLOCK(x)	pthread_rwlock_rdlock(&((cluster *)x)->lock);
#define	CL_ULOCK(x)	pthread_rwlock_unlock(&((cluster *)x)->lock);


/* this struct can be extended when using different metrics,
 * i.e. you may want to add a center when using the cosine measure
 * for ngram vectors */
typedef struct cluster {
	pthread_rwlock_t	lock;
	u_int32_t		cnt;
	u_int32_t		sig_id;
	u_int32_t		sig_rev;
	unsigned long int	threshold;
	void			*parent;
	struct cluster		*prev;
	struct cluster		*next;
	queue			*hq;
} cluster;


inline cluster *cluster_new(void);
void cluster_free(void *cl);

inline cluster *create_hashlist(hash *hl1, hash *hl2);
qelem *add_entry_to_cluster(cluster *cl, hash *entry, const ssize_t max_size);
cluster *create_cluster(hash *l1, hash *l2, const ssize_t max_size);
cluster *clusters_merge(queue *q, cluster *dst, cluster *src);
void clusterlist_delete(cluster *list);

#endif
