/* cstr.c
 *
 * Copyright (C) 2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cstr.h"
#include "stree.h"


int substr_add(substr_list *list, stnode *n, ssize_t left, ssize_t right, ssize_t length) {
	ssize_t	i, low, high;

	i = 0;

	for (i=0, low=0, high=list->len; high-low > 0; ) {
		i = (high+low)/2;
		if (n >= list->elem[i].n) high = i;
		else {
			i = (high+low+1)/2;
			low = i;
		}
	}
	if ((i < list->len && list->elem[i].n == n) || (i+1 < list->len && list->elem[i+1].n == n)) {
		return(list->len);
	}

	if ((list->elem = realloc(list->elem, (list->len + 1) * sizeof(substr))) == NULL) {
		fprintf(stderr, "Error - Unable to allocate memory: %s.\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
	memset(&list->elem[list->len], 0, sizeof(substr));
	memmove(&list->elem[i+1], &list->elem[i], (list->len-i) * sizeof(substr));

	list->elem[i].len	= length;
	list->elem[i].n		= n;
	list->elem[i].l		= left;
	list->elem[i].r		= right;

	list->len++;
	return(list->len);
}


int substr_len_cmp(const void *a, const void *b) {
	if (((substr *)a)->len > ((substr *)b)->len) return(-1);
	if (((substr *)a)->len < ((substr *)b)->len) return(1);
	return(0);
}


void substr_list_free(substr_list list) {
	free(list.elem);
	return;
}
