/* net.c
 *
 * Copyright (C) 2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <arpa/inet.h>
#include <errno.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>

#include "nebula.h"
#include "net.h"

int net_listen(u_int16_t port) {
	int			listen_fd, sockopt;
	struct sockaddr_in	srv;

	listen_fd	= -1;

	// create socket
	if ((listen_fd = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
		fprintf(stderr, "Could not create socket for port 12345: %s.\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
	sockopt = 1;
	if (setsockopt(listen_fd, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof(int)) < 0)
		fprintf(stdout, "Warning - Unable to set SO_REUSEADDR on listening socket.\n");

	// bind port
	memset(&srv, 0, sizeof(struct sockaddr_in));
	srv.sin_family		= AF_INET;
	srv.sin_addr.s_addr	= htonl(INADDR_ANY);
	srv.sin_port		= htons(port);
	if (bind(listen_fd, (struct sockaddr *) &srv, sizeof(srv)) < 0) {
		fprintf(stderr, "Error - Could not bind to port %u/tcp: %s.\n", port, strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	// listen on port
	if (listen(listen_fd, 0) < 0) {
		fprintf(stderr, "Error - Could not listen on port %u/tcp: %s.\n", port, strerror(errno));
		exit(EXIT_FAILURE);
	}

	return(listen_fd);
}


int net_accept(int listen_fd) {
	int			conn_fd;

	if ((conn_fd = accept(listen_fd, (struct sockaddr *) NULL, NULL)) == -1) {
		fprintf(stderr, "Could not accept incoming connection: %s.\n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	return(conn_fd);
}
