/* session.h
 *
 * Copyright (C) 2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __NEBULA_SESSION_H
#define __NEBULA_SESSION_H 1

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <poll.h>
#include <sys/types.h>


// hmac stuff
#define	HMAC_HASH_SIZE	128	// for sha512
#define	HMAC_BLOCK_SIZE	256	// for sha512
#define IPAD_VAL	0x36
#define	OPAD_VAL	0x5C

u_char	k[HMAC_BLOCK_SIZE];
u_char	k_ipad[HMAC_BLOCK_SIZE];
u_char	k_opad[HMAC_BLOCK_SIZE];


// states
typedef enum sstate {
	SS_NEW = 0,
	SS_NONCE_SENT,
	SS_UNAUTHENTICATED,
	SS_AUTHENTICATED,
	SS_REQUEST_SENT,
	SS_PROTOCOL_READ,
	SS_PORT_READ,
	SS_ATTACK_LEN_READ,
	SS_CATTACK_LEN_READ,
	SS_CATTACK_READ,
	SS_HMAC_LEN_READ,
	SS_FINISHED
} sstate;

typedef struct submission {
	sstate		state;			// session state
	u_int32_t	bytes_read;		// number of bytes read so far
	u_int32_t	nonce;			// session nonce, stored in network byte order
	char		secret_hash[129];	// hash(nonce+secret)
	char		*md5sum;		// md5 hash of uncompressed attack data
	unsigned long	attack_len;		// length of uncompressed attack data
	u_int32_t	cattack_len;		// length of compressed attack data

	// the following three fields are HMAC'ed
	u_char		protocol;		// attack protocol
	u_int16_t	port;			// attack port
	u_char		*cattack;		// compressed attack data

	u_int16_t	hmac_len;		// length of HMAC
	char		*hmac;			// HMAC

	u_char		*attack;		// uncompressed attack data
} submission;


void session_reset(submission *s, int session_number);
char *session_hmac(u_char **msg, ssize_t len);
int session_handle_data(struct pollfd *pfd, submission *s);

#endif
