/* sig.h
 *
 * Copyright (C) 2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __NEBULA_SIG_H
#define __NEBULA_SIG_H 1

#include <pthread.h>
#include <stdio.h>

#include "cluster.h"
#include "lca.h"
#include "stree.h"
#include "time.h"


u_int32_t		global_sid;	// incrementing snort sid, must be mutexed

pthread_rwlock_t	sidlock;
pthread_mutex_t		siggen_mutex;
pthread_mutex_t		sigwrite_mutex;

pid_t			snort_pid;
char			*rules_file;


typedef enum sig_type {
	SIG_RAW,
	SIG_SNORT
} sigtype;

typedef enum proto_type {
	SIGPROTO_NONE=0,
	SIGPROTO_TCP=1,
	SIGPROTO_UDP=2,
	SIGPROTO_TCPUDP=4,
	SIGPROTO_INVALID=0xffff
} sigproto_type;

typedef enum port_type {
	SIGPORT_NOTSET,
	SIGPORT_UNIQ,
	SIGPORT_LIST,
	SIGPORT_INTERVAL,
	SIGPORT_ANY
} sigport_type;


typedef struct {
	sigtype		type;
	sigport_type	port_type;
	sigproto_type	proto_type;
	u_int16_t	port;
	cluster		*cl;
	substr_list	cstr_list;
	void		*metainfo;
	time_t		create_time;
} signature;


struct {
	u_int32_t	sid;
	u_int32_t	rev;
	char		*msg;
	char		*classtype;
} snort_info;


typedef struct {
	ssize_t	org_off;	// actual offset in the GST input string
	ssize_t	min_off;	// minimum offset in byte streams
	ssize_t	max_off;	// maximum offset in byte streams
	ssize_t	len;		// segment length
} sseg;

void *pt_pollgq(void *param);
void *gensig(void *cl);
void build_sig(signature *sig, stree *t, lcatbl *lca_table, stnode **leaves, ssize_t num_leaves, ssize_t min_len, double min_ent);
int append_snortsig_to_rulefile(const char *filename, const stree *t, const signature *sig, const sseg *seglist, const ssize_t num_frags);

#endif
