/* stree.h
 *
 * Copyright (C) 2007-2008 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as
 * published by the Free Software Foundation.  You may not use, modify or
 * distribute this program under any other version of the GNU General
 * Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Some of this code is based on Shlomo Yona's suffix tree implementation.
 */

#ifndef __NEBULA_STREE_H
#define __NEBULA_STREE_H 1

#include <sys/types.h>

#include "avl.h"
#include "util.h"


// Error return value for some functions. Initialized  in ST_CreateTree.
u_int32_t    ST_ERROR;


// stree structs

typedef struct stree_node {
	struct stree_node	*sons;			/* A linked list of sons of that node */
	struct stree_node	*right_sibling;		/* A linked list of right siblings of that node */
	struct stree_node	*left_sibling;		/* A linked list of left siblings of that node */
	struct stree_node	*father;		/* A pointer to that node's father */
	struct stree_node	*suffix_link;		/* A pointer to the node that represents the largest suffix of the current node */
	u_int32_t		path_position;		/* Index of the start position of the node's path */
	u_int32_t		edge_label_start;	/* Start index of the incoming edge */
	u_int32_t		edge_label_end;		/* End index of the incoming edge */
	u_int32_t		string_id;		// unique string terminator
	u_int32_t		id;			// id for lca computation
	u_char			flag;
} stnode;

typedef struct stree {
	u_int32_t	e;		/* The virtual end of all leaves */
	lchar		*tree_string;	/* The one and only real source string of the tree. 
					 * All edge-labels contain only indices to this string and do not contain the characters themselves */
	u_int32_t	length;		/* The length of the source string */
	u_int32_t	size;		/* number of nodes */
	stnode		*root;		/* The node that is the head of all others. It has no siblings nor a father */
	u_int32_t	*string_offset;	// list with start offsets of concatenated input strings

} stree;



typedef struct substr {
	ssize_t		len;
	stnode		*n;
	ssize_t		l;	// left child node interval boundary
	ssize_t		r;	// right child node interval boundary
} substr;


typedef struct substr_list {
	ssize_t	len;
	substr	*elem;
} substr_list;


struct lcatbl;

// stree prototypes
stree* ST_CreateTree(void);
inline u_int32_t get_node_label_end(stree *tree, stnode* node);
u_int32_t ST_FindSubstring(stree *tree, lchar* W, u_int32_t P);
void ST_PrintTree(stree *tree, int cs_only);
void ST_DeleteTree(stree* tree);
u_int32_t ST_SelfTest(stree* tree);
stree *ST_AddStringList(stree *tree, lchar *str, u_int32_t length);
void set_ids(stree *t, stnode *n, stnode ***leaves, u_int32_t *num_leaves, u_int32_t *id);
void get_substring_endnodes(stnode *root, substr_list *cstr_list, ssize_t min_length);
void list_substrings(stree *t, stnode *cstr_node, ssize_t length);


#endif
